import { __decorate } from "tslib";
import { bind } from '@roots/bud-support/decorators/bind';
import isFunction from '@roots/bud-support/isFunction';
import Value from '@roots/bud-support/value';
import { Hooks } from '../base/base.js';
/**
 * Asynchronous hooks registry
 */
export class AsyncHooks extends Hooks {
    /**
     * Get a value
     */
    async get(id, fallback) {
        return await [Value.make(fallback), ...(this.store[id] ?? [])]
            .map(Value.get)
            .reduce(async (accumulated, current) => {
            const previous = await accumulated;
            return isFunction(current) ? await current(previous) : current;
        });
    }
    /**
     * Set a value
     */
    set(id, ...input) {
        if (!this.has(id))
            this.store[id] = [];
        input.map(Value.make).map((value) => {
            if (typeof value.get() === `function`)
                this.store[id].push(value);
            else
                this.store[id] = [value];
        });
        return this.app;
    }
    /**
     * Set multiple values
     */
    setRecords(map) {
        Object.entries(map).map(([k, v]) => this.set(k, v));
        return this.app;
    }
}
__decorate([
    bind
], AsyncHooks.prototype, "get", null);
__decorate([
    bind
], AsyncHooks.prototype, "set", null);
__decorate([
    bind
], AsyncHooks.prototype, "setRecords", null);
